# Copyright (C) 2023 p-ban.com Corp.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

import pcbnew
import os

try:
    from .action import *
    from .pban import *
except ImportError:
    import sys
    sys.path.append(os.path.dirname(__file__))
    from action import *
    from pban import *


class PbanQuotePlugin(pcbnew.ActionPlugin):
    def defaults(self):
        self.name = "P-ban Quote Plugin"
        self.category = "Manufacturing"
        self.description = "P板.comの1-Click見積と連携し、KiCADのデータから外形サイズや構成層数などダイレクトに読み込みます。"
        self.show_toolbar_button = True  # Optional, defaults to False
        self.icon_file_name = os.path.join(os.path.dirname(__file__), "action_icon.png")
        self.dark_icon_file_name = self.icon_file_name

    def Run(self):
        try:
            PBanQuotation().open()
        except NotSupportedParams as e:
            pass


PbanQuotePlugin().register()  # Instantiate and register to Pcbnew
