# Copyright (C) 2023 p-ban.com Corp.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from enum import Enum

W_MSG_OPEN = "構成層数、外形寸法の情報が1-Click見積へ展開されます。\nその他詳細設定については1-Click見積にて任意選択をお願いいたします。\n今後のプラグイン更新にご期待ください。"
W_MSG_SIZES = "外形サイズが判別できません"

W_MSG_LAYERS = "14層以上は下記URLよりお問い合わせください"
W_URL_LAYERS = "https://www.p-ban.com/product/form/f_mtl_rigid.html"

W_MSG_BOARDTHICKNESS = "板厚は、0.15mm, 0.2mm, 0.3mm, 0.4mm, 0.5mm, 0.6mm, 0.8mm, 1.0mm, 1.2mm, 1.6mm, 1.8mm, 2.0mm, 2.4mm, 3.0mm から選択可能です。 1.8mm は4層以上で構成時時に選択可能です。"
W_URL_BOARDTHICKNESS = ""

W_MSG_PATTERNS = "最小パターン幅/間隙が0.075mm未満です\n下記URLよりお問い合わせください"
W_URL_PATTERNS = "https://www.p-ban.com/form/contact_form.html"

W_MSG_LAND = "最小ビア径/ランド径がφ0.15/0.35mm未満です\n下記URLよりお問い合わせください\n"
W_URL_LAND = "https://www.p-ban.com/form/contact_form.html"


class NotSupportedParams(Exception):
    pass


class KibanType(Enum):
    RIGID = 1
    FPC   = 4
    ALM   = 5


class Layer(Enum):
    # Rigid
    RIGID_BOTHSIDES = 1
    RIGID_ONESIDE   = 3
    RIGID_FOUR      = 2
    RIGID_SIX       = 4
    RIGID_EIGHT     = 5
    RIGID_TEN       = 10
    RIGID_TWELVE    = 11
    # Flexible
    FPC_ONESIDE   = 6
    FPC_BOTHSIDES = 7
    # Alminium
    ALM_ONESIDE   = 8
    ALM_TWO       = 9

    @staticmethod
    def choice(cnt, _type=KibanType.RIGID):
        if _type == KibanType.RIGID:
            if cnt == 1:
                return Layer.RIGID_ONESIDE
            elif cnt == 2:
                return Layer.RIGID_BOTHSIDES
            elif cnt == 4:
                return Layer.RIGID_FOUR
            elif cnt == 6:
                return Layer.RIGID_SIX
            elif cnt == 8:
                return Layer.RIGID_EIGHT
            elif cnt == 10:
                return Layer.RIGID_TEN
            elif cnt == 12:
                return Layer.RIGID_TWELVE
            else:
                raise NotSupportedParams()

        elif _type == KibanType.FPC:
            if cnt == 1:
                return Layer.FPC_ONESIDE
            elif cnt == 2:
                return Layer.FPC_BOTHSIDES
            else:
                raise NotSupportedParams()

        elif _type == KibanType.ALM:
            if cnt == 1:
                return Layer.ALM_ONESIDE
            elif cnt == 2:
                return Layer.ALM_TWO
            else:
                raise NotSupportedParams()


class PatternSide(Enum):
    PTN_SD_L1_L2 = 2047200000
    PTN_SD_L1_L1 = 2047200001
    PTN_SD_L2_L1 = 2047200002
    PTN_SD_L2_L2 = 2047200003


class MinPattern(Enum):
    MIN_PTN_0127 = (2041600000, [KibanType.RIGID, KibanType.FPC, KibanType.ALM], 0.127)
    MIN_PTN_003  = (2041600009, [KibanType.FPC], 0.03)
    MIN_PTN_005  = (2041600008, [KibanType.FPC], 0.05)
    MIN_PTN_006  = (2041600007, [KibanType.FPC], 0.06)
    MIN_PTN_0075 = (2041600003, [KibanType.RIGID, KibanType.FPC], 0.075)
    MIN_PTN_010  = (2041600002, [KibanType.RIGID, KibanType.FPC, KibanType.ALM], 0.10)
    MIN_PTN_015  = (2041600001, [KibanType.RIGID, KibanType.ALM], 0.15)
    MIN_PTN_025  = (2041600004, [KibanType.RIGID, KibanType.FPC, KibanType.ALM], 0.25)
    MIN_PTN_030  = (2041600005, [KibanType.RIGID, KibanType.FPC, KibanType.ALM], 0.30)
    MIN_PTN_035  = (2041600006, [KibanType.RIGID, KibanType.FPC, KibanType.ALM], 0.35)

    @staticmethod
    def pattern(width, kiban_type=KibanType.RIGID):
        if kiban_type == KibanType.RIGID:
            if width < 0.075:
                raise NotSupportedParams()
            elif width < 0.1:
                return MinPattern.MIN_PTN_0075
            elif width < 0.127:
                return MinPattern.MIN_PTN_010
            elif width < 0.15:
                return MinPattern.MIN_PTN_0127
            elif width < 0.25:
                return MinPattern.MIN_PTN_015
            elif width < 0.30:
                return MinPattern.MIN_PTN_025
            elif width < 0.35:
                return MinPattern.MIN_PTN_030
            else:
                return MinPattern.MIN_PTN_035

        elif kiban_type == KibanType.FPC:
            if width < 0.03:
                raise NotSupportedParams()
            elif width < 0.05:
                return MinPattern.MIN_PTN_003
            elif width < 0.06:
                return MinPattern.MIN_PTN_005
            elif width < 0.075:
                return MinPattern.MIN_PTN_006
            elif width < 0.1:
                return MinPattern.MIN_PTN_0075
            elif width < 0.127:
                return MinPattern.MIN_PTN_010
            elif width < 0.25:
                return MinPattern.MIN_PTN_0127
            elif width < 0.30:
                return MinPattern.MIN_PTN_025
            elif width < 0.35:
                return MinPattern.MIN_PTN_030
            else:
                return MinPattern.MIN_PTN_035

        elif kiban_type == KibanType.ALM:
            if width < 0.1:
                raise NotSupportedParams()
            elif width < 0.127:
                return MinPattern.MIN_PTN_010
            elif width < 0.15:
                return MinPattern.MIN_PTN_0127
            elif width < 0.25:
                return MinPattern.MIN_PTN_015
            elif width < 0.30:
                return MinPattern.MIN_PTN_025
            elif width < 0.35:
                return MinPattern.MIN_PTN_030
            else:
                return MinPattern.MIN_PTN_035

    def __init__(self, value, kiban_types, width):
        self._value_ = value
        self.kiban_types = kiban_types
        self.width = width

    def validate(self, kiban_type):
        if kiban_type not in self.kiban_types:
            # raise NotSupportedParams("Invalid kiban type")
            return False

        return True


class MinLand(Enum):
    # Rigid
    MIN_LND_RGD_03_06   = 2042600000
    MIN_LND_RGD_03_05   = 2042600001
    MIN_LND_RGD_025_05  = 2042600002
    MIN_LND_RGD_02_05   = 2042600003
    MIN_LND_RGD_02_045  = 2042600004
    MIN_LND_RGD_015_04  = 2042600005
    MIN_LND_RGD_015_035 = 2042600006

    # FPC
    MIN_LND_FPC_03_06   = 2049600000
    MIN_LND_FPC_025_05  = 2049600001
    MIN_LND_FPC_02_05   = 2049600002
    MIN_LND_FPC_015_03  = 2049600003
    MIN_LND_FPC_012_027 = 2049600004

    # ALM
    MIN_LND_ALM_03_06   = 2046700000

    @staticmethod
    def choice(hole, size, kiban_type=KibanType.RIGID):
        if kiban_type == KibanType.RIGID:
            if hole < 0.15 or size < 0.35:
                raise NotSupportedParams()
            elif hole < 0.2:
                if size < 0.4:
                    return MinLand.MIN_LND_RGD_015_035
                elif size >= 0.4:
                    return MinLand.MIN_LND_RGD_015_04
            elif hole < 0.25:
                if size < 0.45:
                    return MinLand.MIN_LND_RGD_015_04
                elif size < 0.5:
                    return MinLand.MIN_LND_RGD_02_045
                elif size >= 0.5:
                    return MinLand.MIN_LND_RGD_02_05
            elif hole < 0.3:
                if size < 0.5:
                    return MinLand.MIN_LND_RGD_02_045
                elif size >= 0.5:
                    return MinLand.MIN_LND_RGD_025_05
            else:
                if size < 0.5:
                    return MinLand.MIN_LND_RGD_02_045
                elif size < 0.6:
                    return MinLand.MIN_LND_RGD_03_05
                else:
                    return MinLand.MIN_LND_RGD_03_06

        elif kiban_type == KibanType.FPC:
            if hole < 0.12 or size < 0.27:
                raise NotSupportedParams()
            elif hole < 0.15:
                return MinLand.MIN_LND_FPC_012_027
            elif hole < 0.2:
                if size < 0.3:
                    return MinLand.MIN_LND_FPC_012_027
                else:
                    return MinLand.MIN_LND_FPC_015_03
            elif hole < 0.25:
                if size < 0.3:
                    return MinLand.MIN_LND_FPC_012_027
                elif size < 0.5:
                    return MinLand.MIN_LND_FPC_015_03
                else:
                    return MinLand.MIN_LND_FPC_02_05
            elif hole < 0.3:
                if size < 0.3:
                    return MinLand.MIN_LND_FPC_012_027
                elif size < 0.5:
                    return MinLand.MIN_LND_FPC_015_03
                else:
                    return MinLand.MIN_LND_FPC_025_05
            else:
                if size < 0.3:
                    return MinLand.MIN_LND_FPC_012_027
                elif size < 0.5:
                    return MinLand.MIN_LND_FPC_015_03
                elif size < 0.6:
                    return MinLand.MIN_LND_FPC_025_05
                return MinLand.MIN_LND_FPC_03_06

        elif kiban_type == KibanType.ALM:
            if hole < 0.3 or size < 0.6:
                raise NotSupportedParams()
            else:
                return MinLand.MIN_LND_ALM_03_06

        raise NotSupportedParams("Not supported land(or via) size")


class BoardThickness(Enum):

    BRDTHICKNESS_0_15 = 2040600011
    BRDTHICKNESS_0_2  = 2040600012
    BRDTHICKNESS_0_3  = 2040600013
    BRDTHICKNESS_0_4  = 2040600007
    BRDTHICKNESS_0_6  = 2040600008
    BRDTHICKNESS_0_8  = 2040600001
    BRDTHICKNESS_1_0  = 2040600002
    BRDTHICKNESS_1_2  = 2040600003
    BRDTHICKNESS_1_6  = 2040600000
    BRDTHICKNESS_1_8  = 2040600005 # available if gte 4 layers
    BRDTHICKNESS_2_0  = 2040600006
    BRDTHICKNESS_2_4  = 2040600009
    BRDTHICKNESS_3_0  = 2040600010

    @staticmethod
    def choice(thickness, layercount):
        supported = [0.15, 0.2, 0.3, 0.4, 0.6, 0.8, 1.0, 1.2, 1.6, 1.8, 2.0, 2.4, 3.0 ]
        if not thickness in supported:
            raise NotSupportedParams("invalid board thickness")
        if thickness == 1.8 and layercount < 4:
            raise NotSupportedParams("invalid board thickness")

        return [ BoardThickness.BRDTHICKNESS_0_15,
                 BoardThickness.BRDTHICKNESS_0_2,
                 BoardThickness.BRDTHICKNESS_0_3,
                 BoardThickness.BRDTHICKNESS_0_4,
                 BoardThickness.BRDTHICKNESS_0_6,
                 BoardThickness.BRDTHICKNESS_0_8,
                 BoardThickness.BRDTHICKNESS_1_0,
                 BoardThickness.BRDTHICKNESS_1_2,
                 BoardThickness.BRDTHICKNESS_1_6,
                 BoardThickness.BRDTHICKNESS_1_8,
                 BoardThickness.BRDTHICKNESS_2_0,
                 BoardThickness.BRDTHICKNESS_2_4,
                 BoardThickness.BRDTHICKNESS_3_0][supported.index(thickness)]


class Color(Enum):
    GREEN     = 1
    RED       = 2
    BLUE      = 3
    YELLOW    = 4
    WHITE     = 5
    BLACK     = 6
    BLACK_MAT = 7
    PURPLE    = 8


class ResistPrint(Enum):
    RP_BOTHSIDES = 1
    RP_TOP       = 2
    RP_BOTTOM    = 3
    RP_NONE      = 4


class ResistColor(Enum):
    RC_RGD_GREEN     = 2040500000
    RC_RGD_RED       = 2040500001
    RC_RGD_BLUE      = 2040500002
    RC_RGD_YELLOW    = 2040500003
    RC_RGD_WHITE     = 2040500004
    RC_RGD_BLACK     = 2040500005
    RC_RGD_BLACK_MAT = 2040500006
    RC_RGD_PURPLE    = 2040500007

    RC_FPC_WHITE       = 2046800000
    RC_FPC_GREEN       = 2046800001
    RC_FPC_RED         = 2046800002
    RC_FPC_BLUE        = 2046800003
    RC_FPC_YELLOW      = 2046800004
    RC_FPC_BLACK       = 2046800005
    RC_FPC_BLACK_MAT   = 2046800006


class SilkColor(Enum):
    SLK_RGD_WHITE    = 2040900000
    SLK_RGD_BLACK    = 2040900001
    SLK_RGD_YELLOW   = 2040900002
    SLK_ALM_WHITE    = 2046900001
    SLK_ALM_BLACK    = 2040900000
    SLK_ALM_YELLOW   = 2040900002
    SLK_FPC_WHITE    = 2049100000
    SLK_FPC_BLACK    = 2049100001
